/** @file addidata_dataclient.h
* @brief data client related utility functions
*
* @author S.Nahas
* @date Jan 28, 2009
*
* @version  $LastChangedRevision: 3021 $
* @version $LastChangedDate: 2008-06-11 10:16:10 +0200 (Wed, 11 Jun 2008) $
*
* @par LICENCE
* @verbatim
   Copyright (C) 2009  ADDI-DATA GmbH for the source code of this module.

        ADDI-DATA GmbH
        Airport boulevard B210
        D-77836 Rheinmuenster
        Tel: +49(0)7229/1847-0
        Fax: +49(0)7229/1847-200
        http://www.addi-data-com
        info@addi-data.com

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   You shoud also find the complete GPL in the COPYING file
   accompanying this source code.
   @endverbatim
*
*
*/


#ifndef ADDIDATA_DATACLIENT_H_
#define ADDIDATA_DATACLIENT_H_

#include <stdio.h>

#ifdef WIN32
	#include <stdint_compat.h>
	#define __INLINE__ __inline
#else
	#include <stdint.h>
	#define __INLINE__ inline
#endif

/** establish a connection to the data server
 * of module "name" at TCP port "port"
 *
 * @param[in] name A non-null string containing the IP address of the module to contact
 * @param[in] port - may be null, in this case the default port is used (8989)
 * @param[out] fd :file descriptor to the socket
 * @retval 0 success
 * @retval -1 socket() failed, use addidata_network_perror() to get a human-readable explanation
 * @retval -2 connect() failed, use addidata_network_perror() to get a human-readable explanation
 * @retval -3 could not parse address "name"
 *
 * @note under MS Windows, a call to addidata_network_init() must have been done before calling this function
 */
int addidata_dataclient_tcp_connect(const char* name, unsigned int port, int* fd);

/** multi-platform wrapper around recv()
 *
 * please refer to your system documentation for more information
 *
 * @retval 0 the peer has performed an orderly shutdown.
 * @retval -1 an error occurred, use addidata_network_perror() to get a human-readable explanation
 * @retval otherwise number of bytes received
 */
int addidata_dataclient_recv(int fd, uint32_t data[], int count, int flags);


/** receives "count" bytes from data server "fd"
 *
 * addidata_dataclient_recvall() tries very hard to receive the requested amount of data
 * and block until it is present or an error occurs.
 *
 * @param[in] fd : a value returned by addidata_dataclient_tcp_connect()
 * @param[in,out] data : buffer to store read data in
 * @param[in] count : number of uint32_t to read
 *
 * @retval 0 the peer has performed an orderly shutdown.
 * @retval -1 an error occurred, use addidata_network_perror() to get a human-readable explanation
 * @retval otherwise number of bytes received
 *
 * @note data returned by the MSX-E are all multiple of 4 bytes
 * @note a call to addidata_network_connect() must have been done before calling this function
 *
 */
int addidata_dataclient_recvall(int fd, uint32_t data[], int count);


/** receives "count" bytes from data server "fd" in non-blocking mode.
 *
 * addidata_dataclient_recvall_ifavailable() tests the amount of data available. if data available, then read it.
 *
 * @param[in] fd : a value returned by addidata_dataclient_tcp_connect()
 * @param[in,out] data : buffer to store read data in
 * @param[in] count : number of uint32_t to read
 *
 * @retval 0 no data available.
 * @retval -1 an error occurred, use addidata_network_perror() to get a human-readable explanation
 * @retval otherwise number of bytes received
 *
 * @note data returned by the MSX-E are all multiple of 4 bytes
 * @note a call to addidata_network_connect() must have been done before calling this function
 *
 * @warning Not ported on Microsoft Windows yet!
 */
int addidata_dataclient_recvall_ifavailable(int fd,uint32_t data[], int count);

/** close connection to data server "fd"
 * @param[in] fd a value returned by addidata_dataclient_tcp_connect()
 *
 * @retval 0
 * */
int addidata_dataclient_disconnect(int fd);

/** format for the data parsing function
 *
 * %%f: float 32 bit - use default %%f \n
 * %%u: 32 bits unsigned  - use default %%u \n
 * %%d: 32 bits signed - use default %%d \n
 * %%x: 32 bits signed, hexa - use default %%x \n
 * %%X: 32 bits signed, hexa - use default %%X \n
 * %%y: swallows value \n
 * ' ': (space) separator \n
 *
 * all other characters are output as it.
 */

/** from a format string, returns the expected amount of 32 bits read from data server
 * @retval -1 parse error
 */
int addidata_dataclient_fmtcount(const char *format);

/** function similar to fprintf()
 *
 * @param[in,out] stream output
 * @param[in] format to use
 * @param[in] data pointer the array of values
 * @param[in] count size of the array "data" in 32 bits units
 *
 * @retval >0 number of converted values
 * @retval -1 error in format
 * @retval -2 format overflow "count"
 *
 * @note conversion is stopped when an error in format is encountered
 * */
int addidata_dataclient_fprintf(FILE *stream, const char *format, uint32_t data[], int count);

/** equivalent to addidata_dataclient_fprintf(stdout, format, data, count) */
int addidata_dataclient_printf(const char *format, uint32_t data[], int count);

#define DEFAULT_DATASERVER_PORT (8989)

#endif /* ADDIDATA_DATACLIENT_H_ */

